@@ -15,6 +15,8 @@
 
 from oslo_config import cfg
 
+os_win_group = 'os_win'
+
 os_win_opts = [
     cfg.StrOpt('hbaapi_lib_path',
                default='hbaapi.dll',
@@ -27,7 +29,16 @@ os_win_opts = [
                      'almost all operations require a reference to a '
                      'switch port. The cached objects are no longer valid '
                      'if the VM they are associated with is destroyed.'),
+    cfg.IntOpt('wmi_job_terminate_timeout',
+               default=120,
+               help='The default amount of seconds to wait when stopping '
+                    'pending WMI jobs. Setting this value to 0 will '
+                    'disable the timeout.'),
 ]
 
 CONF = cfg.CONF
-CONF.register_opts(os_win_opts, 'os_win')
+CONF.register_opts(os_win_opts, os_win_group)
+
+
+def list_opts():
+    return [(os_win_group, os_win_opts)]
